using System;
using System.IO;
using System.Text;

namespace DictionaryParser {

    public class Dictionary {

        private const String wordsFileName               = "Words.txt";
        private const String descriptionsFileName        = "Descriptions.txt";
        private const String wordsOffsetsFileName        = "WordsOffsets.txt";
        private const String descriptionsOffsetsFileName = "DescriptionsOffsets.txt";
        private const long splitDescriptionsFileSize = 0x4000;
        private long wordsFileSize;
        private long descriptionsFileSize;
        private long wordsOffsetsFileSize;
        private long descriptionsOffsetsFileSize;
        private int minWordSize;
        private int maxWordSize;
        private int minDescriptionSize;
        private int maxDescriptionSize;
        private int minWordSizeIndex;
        private int maxWordSizeIndex;
        private int minDescriptionSizeIndex;
        private int maxDescriptionSizeIndex;
        private String minWordSizeWord;
        private String maxWordSizeWord;
        private String minDescriptionSizeWord;
        private String maxDescriptionSizeWord;
        private int count;

        public Dictionary() {
        }

        public void parse(String fileName) {
            count = 0;
            StreamReader streamReader = new StreamReader(fileName, Encoding.Default);
            StreamWriter wordsStream               = new StreamWriter(Path.GetDirectoryName(fileName) + '\\' + wordsFileName,        false, Encoding.Default);
            StreamWriter descriptionsStream        = new StreamWriter(Path.GetDirectoryName(fileName) + '\\' + descriptionsFileName, false, Encoding.Default);
            FileStream   wordsOffsetsStream        = new FileStream  (Path.GetDirectoryName(fileName) + '\\' + wordsOffsetsFileName,        FileMode.Create);
            FileStream   descriptionsOffsetsStream = new FileStream  (Path.GetDirectoryName(fileName) + '\\' + descriptionsOffsetsFileName, FileMode.Create);
            minWordSize = Int32.MaxValue;
            maxWordSize = 0;
            minDescriptionSize = Int32.MaxValue;
            maxDescriptionSize = 0;
            minWordSizeIndex = -1;
            maxWordSizeIndex = -1;
            minDescriptionSizeIndex = -1;
            maxDescriptionSizeIndex = -1;
            minWordSizeWord = "";
            maxWordSizeWord = "";
            minDescriptionSizeWord = "";
            maxDescriptionSizeWord = "";
            String line;
            int wordsOffset = 0;
            int descriptionsOffset = 0;
            while ((line = streamReader.ReadLine()) != null) {
                if (line != "") {
                    int index = line.IndexOf('|');
                    String word = line.Substring(0, index);
                    String description = line.Substring(index + 1);
                    wordsStream.Write(word + '\n');
                    descriptionsStream.Write(description + '\n');
                    wordsOffsetsStream.WriteByte((byte)((wordsOffset >> 0x18) & 0xFF));
                    wordsOffsetsStream.WriteByte((byte)((wordsOffset >> 0x10) & 0xFF));
                    wordsOffsetsStream.WriteByte((byte)((wordsOffset >> 0x08) & 0xFF));
                    wordsOffsetsStream.WriteByte((byte)((wordsOffset >> 0x00) & 0xFF));
                    descriptionsOffsetsStream.WriteByte((byte)((descriptionsOffset >> 0x18) & 0xFF));
                    descriptionsOffsetsStream.WriteByte((byte)((descriptionsOffset >> 0x10) & 0xFF));
                    descriptionsOffsetsStream.WriteByte((byte)((descriptionsOffset >> 0x08) & 0xFF));
                    descriptionsOffsetsStream.WriteByte((byte)((descriptionsOffset >> 0x00) & 0xFF));
                    if (word.Length < minWordSize) {
                        minWordSize = word.Length;
                        minWordSizeIndex = count;
                        minWordSizeWord = word;
                    }
                    if (word.Length > maxWordSize) {
                        maxWordSize = word.Length;
                        maxWordSizeIndex = count;
                        maxWordSizeWord = word;
                    }
                    if (description.Length < minDescriptionSize) {
                        minDescriptionSize = description.Length;
                        minDescriptionSizeIndex = count;
                        minDescriptionSizeWord = word;
                    }
                    if (description.Length > maxDescriptionSize) {
                        maxDescriptionSize = description.Length;
                        maxDescriptionSizeIndex = count;
                        maxDescriptionSizeWord = word;
                    }
                    wordsOffset += word.Length + 1;
                    descriptionsOffset += description.Length + 1;
                    count++;
                }
            }
            wordsStream.Flush();
            descriptionsStream.Flush();
            wordsOffsetsStream.Flush();
            descriptionsOffsetsStream.Flush();
            wordsFileSize = wordsStream.BaseStream.Length;
            descriptionsFileSize = descriptionsStream.BaseStream.Length;
            wordsOffsetsFileSize = wordsOffsetsStream.Length;
            descriptionsOffsetsFileSize = descriptionsOffsetsStream.Length;
            wordsStream.Close();
            descriptionsStream.Close();
            wordsOffsetsStream.Close();
            descriptionsOffsetsStream.Close();
            streamReader.Close();
            split(Path.GetDirectoryName(fileName) + '\\' + descriptionsFileName, splitDescriptionsFileSize);
        }

        private void split(String filePath, long splitFileSize) {
            FileStream fileStream = new FileStream(filePath, FileMode.Open);
            long fileSize = fileStream.Length;
            int intParts = (int)(fileSize / splitFileSize);
            long remainFileSize = fileSize - (intParts * splitFileSize);
            int i;
            FileStream splitFileStream;
            byte[] data;
            for (i = 0; i < intParts; i++) {
                splitFileStream = new FileStream(Path.GetDirectoryName(filePath) + '\\' + Path.GetFileName(filePath)[0] + Convert.ToString(i) + ".txt", FileMode.Create);
                data = new byte[(int)splitFileSize];
                fileStream.Read(data, 0, (int)splitFileSize);
                splitFileStream.Write(data, 0, (int)splitFileSize);
                splitFileStream.Close();
            }
            splitFileStream = new FileStream(Path.GetDirectoryName(filePath) + '\\' + Path.GetFileName(filePath)[0] + Convert.ToString(i) + ".txt", FileMode.Create);
            data = new byte[(int)remainFileSize];
            fileStream.Read(data, 0, (int)remainFileSize);
            splitFileStream.Write(data, 0, (int)remainFileSize);
            splitFileStream.Close();
            fileStream.Close();
        }

        public String getWordsFileName() {
            return wordsFileName;
        }

        public String getDescriptionsFileName() {
            return descriptionsFileName;
        }

        public String getWordsOffsetsFileName() {
            return wordsOffsetsFileName;
        }

        public String getDescriptionsOffsetsFileName() {
            return descriptionsOffsetsFileName;
        }

        public long getWordsFileSize() {
            return wordsFileSize;
        }

        public long getDescriptionsFileSize() {
            return descriptionsFileSize;
        }

        public long getWordsOffsetsFileSize() {
            return wordsOffsetsFileSize;
        }

        public long getDescriptionsOffsetsFileSize() {
            return descriptionsOffsetsFileSize;
        }

        public int getMinWordSize() {
            return minWordSize;
        }

        public int getMaxWordSize() {
            return maxWordSize;
        }

        public int getMinDescriptionSize() {
            return minDescriptionSize;
        }

        public int getMaxDescriptionSize() {
            return maxDescriptionSize;
        }

        public int getMinWordSizeIndex() {
            return minWordSizeIndex;
        }

        public int getMaxWordSizeIndex() {
            return maxWordSizeIndex;
        }

        public int getMinDescriptionSizeIndex() {
            return minDescriptionSizeIndex;
        }

        public int getMaxDescriptionSizeIndex() {
            return maxDescriptionSizeIndex;
        }

        public String getMinWordSizeWord() {
            return minWordSizeWord;
        }

        public String getMaxWordSizeWord() {
            return maxWordSizeWord;
        }

        public String getMinDescriptionSizeWord() {
            return minDescriptionSizeWord;
        }

        public String getMaxDescriptionSizeWord() {
            return maxDescriptionSizeWord;
        }

        public int getCount() {
            return count;
        }

    }

}
